package Text::BibTeX::CVSort;

use Text::BibTeX::BibSort;
use Text::BibTeX::CV;
use Carp qw(croak);
@ISA=qw(Text::BibTeX::BibSort);

my $debug=0;

my %month_hash=(
		Jan=>1,
		Feb=>2,
		Mar=>3,
		Apr=>4,
		May=>5,
		Jun=>6,
		Jul=>7,
		Aug=>8,
		Sep=>9,
		Oct=>10,
		Nov=>11,
		Dec=>12,
		Fal=>1,
		Win=>2);

sub munge_title{
  my $str=shift;
  $str=~ tr[a-z][zyxwvutsrqponmlkjihgfedcba];
  $str=~ tr[0-9][9876543210];
  return $str;
}
sub munge_month{
  my $flip=shift;
  my $str=shift || '   ';

  
  my $int=$month_hash{substr($str,0,3)} || -1;
  return sprintf("%02d",$flip ? 12-$int : $int);
}
		

sub sort_endyear{
  my $self=shift;
  my $year=  $self->get('endyear') || $self->get('year');
  $year='9999' if ($year =~ /^[\s\{\}]*$/);
#  print STDERR "endyear=<$year>\n" if $debug;
  return $year;
}

sub sort_key
{
   my $self = shift;
   my ($sortby, $type, $nkey, $skey);

   $sortby = $self->structure->get_options ('sortby');
   croak ("BibSort::sort_key: sortby option is 'none'")
      if $sortby eq 'none';
   croak ("BibSort::sort_key: unknown sortby option '$sortby'")
      unless $sortby eq 'name' || $sortby eq 'year';

   $type = $self->type;

   if ($type eq 'book' || $type eq 'inbook' || $type eq 'article')
   {
      $nkey = $self->format_alt_fields ('author' => 'sort_format_names',
                                        'editor' => 'sort_format_names',
                                        'key'    => 'sortify');
   }
   elsif ($type eq 'proceedings')
   {
      $nkey = $self->format_alt_fields ('editor' => 'sort_format_names',
                                        'organization' => 'sort_format_org',
                                        'key'    => 'sortify');
   }
   elsif ($type eq 'manual')
   {
      $nkey = $self->format_alt_fields ('author' => 'sort_format_names',
                                        'organization' => 'sort_format_org',
                                        'key'    => 'sortify');
   }
#    elsif ($type eq 'interval')
#      {
#       $nkey = $self->format_alt_fields ('title' => 'sortify',
#                                         'key'    => 'sortify');
#     }
   else
   {
      $nkey = $self->format_alt_fields ('author' => 'sort_format_names',
                                        'key'    => 'sortify');
   }

   my $ykey = Text::BibTeX::BibSort::change_case ('l', (Text::BibTeX::BibSort::purify_string ($self->sort_endyear .'    '.$self->get ('year'))));
#   print STDERR $self->key," ykey=$ykey\n" if $debug;

   if (!$self->get_option('omit_month')){
     $ykey .= munge_month($self->get_option($type=>'flip_month'),
			  $self->get('month'));
   }

   $nkey ||= '';
   $skey = ($sortby eq 'name') 
      ? $nkey . '    ' . $ykey
      : $ykey . '    ' . $nkey;

   if (defined($self->get('title'))) {
     $skey .= '    ' .  munge_title($self->sort_format_title ('title'));
   }

#   print STDERR "sort key: $skey";
   return $skey;

}  # sort_key




1;
