package Text::BibTeX::Formatter;

use strict;



sub new{
  my $class = shift;
  my %opts=@_;

  my $self  = {};
  $self->{SECTION}   = $opts{SECTION} || die "SECTION is mandatory";
  $self->{COUNT}   = 0;
  $self->{LASTLABEL} = "";
  bless ($self, $class);
  return $self;
};


# borrowed from btformat
sub format{
  my $self=shift;
  my $entry=shift;

  my (@blocks, $block, $sentence);

  @blocks = $entry->format;
  @blocks = grep ($_, @blocks);        # strip empty blocks

  my @blocksout;

 BLOCK:
  for $block (@blocks)
    {
      my @sentencelist;
      if (ref $block eq 'ARRAY'){
	@sentencelist=@$block;
      } else {
	@sentencelist=($block);
      }
	
      SENTENCE:
      for $sentence (@sentencelist)
      {
         # If sentence has multiple clauses, process them: first, strip
         # out empties, and jump to the next sentence if it turns out 
         # this one is empty (ie. just a bunch of empty clauses).  Then
         # join the left-over clauses with commas.
         if (ref $sentence eq 'ARRAY')
         {
            @$sentence = grep ($_, @$sentence);
            ($sentence = '', next SENTENCE) unless @$sentence;
            $sentence = join (', ', @$sentence);
         }
	 $sentence ||= '';
         # finish sentence with a period if it's not already punctuated
         $sentence .= '.' unless $sentence eq '' || $sentence =~ /([.!?]|\%NOPERIOD\%)$/;
      }

      # Now join together all the sentences in the block, first stripping
      # any empties.
      @sentencelist = grep ($_, @sentencelist);
      next BLOCK unless scalar (@sentencelist);
      $block = join (' ', @sentencelist);     # put the sentences together
      push(@blocksout,$self->post_process($block));
   }

   if (@blocksout)
   {
      return $self->header($entry).
	join ($self->section->markup('inter_block'), @blocksout);
    } else {
      return "";
    }
  
 }

# take care of some hard-coded TeXisms
sub post_process($$){
  my $self=shift;
  my $string=shift;
  if (my $tilde=$self->section->markup('tilde')){
    $string=~s|([^/])\~|$1$tilde|g;
    $string=~s|%NOPERIOD%||g;
  }
  return $string;
}
sub section{
  my $self=shift;
  return $self->{SECTION};
}

sub style{
  my $self=shift;
  return $self->section->style;
}


sub header($$){
  my $self=shift;
  my $entry=shift;
  
  my $style=$self->style;

  my $key = $entry->key;
  my $type = $entry->type;

  my $label;
  
  if ($style eq 'dated'){
    my @dates=();
    if (
	$entry->structure->get_suboption(bibitem=>'omit_month')||
	$entry->structure->get_suboption($type=>'omit_month')
       ){
      my $year=$entry->get('year');
      push(@dates,$year);
      my $endyear=$entry->get('endyear') || $year;

      push(@dates,$endyear) if ($endyear ne $year);
    } else {
      @dates=($entry->format_date, $entry->format_enddate);

    }
    $label=join(" -- ",grep($_, @dates));
#    print STDERR "formatting $key $label $style\n";
      

  } else {
    if ($entry->structure->get_suboption(bibitem=>'number_within_sec')){
      $label= sprintf('%s.%d',$self->section->markup('seccounter_'.
						       $self->section->level),				       ++($self->{COUNT}));
    } else {
      $label = sprintf('%d',++($self->{COUNT}))
    }
  }
#  print STDERR "using label $label\n";
  if ($entry->structure->get_suboption(bibitem=>'compress_labels') &&
      $label eq $self->{LASTLABEL}){
    $label="";
  } else {
    $self->{LASTLABEL}=$label;
  }

  my $header = $self->section->markup('pre_entry');
  $header =~ s/%KEY%/$key/g;
  $header =~ s/%LABEL%/$label/g;
  $header =~ s/%STYLE%/$style/g;
  return $header;
}



sub start_list{
  my $self=shift;

  my $rval=$self->section->markup('blist');

  my $widest=$self->{SECTION}->widestlabel;
  $rval=~ s/%WIDEST%/$widest/g;

  return $rval;
}
sub end_list{
  my $self=shift;

  return $self->section->markup('elist');
}

sub emptysection{
  my $self=shift;
  my $level=$self->section->level;
  if ($self->section->structure->get_options('emptysections')) {
    return $self->section->markup('emptysec_'.$level);
  } else {
    return '';
  }
}
sub secheader{
  my $self=shift;
  my $level=$self->section->level;

  my $sec=$self->section->markup('section_'.$level);


  my $title=$self->section->title;
  my $key=$self->section->key;

  $sec =~ s/%TITLE%/$title/g;
  $sec =~ s/%KEY%/$key/g;
  my $annote=$self->section->annote;
  if ($annote){
    $sec .= $self->section->markup('annote');
    $sec =~ s/%ANNOTE%/$annote/g;
  }
  return $sec;
}
1;
